/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef FPSCOUNTER_H
#define FPSCOUNTER_H

#ifndef ENABLE_FPS
#define ENABLE_FPS 0
#endif

#if ENABLE_FPS

#include "time.h"
#include "Font.h"
#include "Shaders.h"
#include "Common.h"
#include "glm/gtx/transform.hpp"
#include "glm/gtc/matrix_inverse.hpp"
#include <glm/gtc/type_ptr.hpp>
#ifdef _WIN32
#include <windows.h>
#endif
#ifdef _linux_
#include <sys/time.h>
#endif
#define WRITEFPS2FILE 1
#if WRITEFPS2FILE
#include <iostream>
#include <iomanip>
#include <fstream>
#endif
#define TIMETHRESHOLD 30
#define FPS_FILE      "./FPSCounter.txt"
#define VERTSHADER "#version 330\n"\
"uniform struct Matrices\n"\
"{\n"\
"    mat4 projMatrix;\n"\
"    mat4 modelViewMatrix;\n"\
"} matrices;\n"\
"layout (location = 0) in vec2 inPosition;\n"\
"layout (location = 1) in vec2 inCoord;\n"\
"out vec2 texCoord;\n"\
"void main()\n"\
"{\n"\
"    gl_Position = matrices.projMatrix * matrices.modelViewMatrix *\n"\
"                    vec4(inPosition, 0.0, 1.0);\n"\
"    texCoord = inCoord;\n"\
"}"
#define FRAGSHADER "#version 330\n"\
"in vec2 texCoord;\n"\
"out vec4 outputColor;\n"\
"uniform sampler2D gSampler;\n"\
"uniform vec4 vColor;\n"\
"void main()\n"\
"{\n"\
"    vec4 vTexColor = texture2D(gSampler, texCoord);\n"\
"    outputColor = vec4(vTexColor.r, vTexColor.r, vTexColor.r,"\
"                  vTexColor.r)*vColor;\n"\
"}"

using namespace glTF;

class FPSCounter
{
public:
    void timeStamp();

    void printFPS(glTFViewport* pViewpoit);
    bool loadFPSShader(ShaderProgram* pShaderProgram);

    FPSCounter(unsigned char* fontBuf,unsigned int bufSize,
               ShaderProgram* pShaderProgram)
        : mFPS(0), mDurationTime(0),mCurrentTime(0)
        ,mLastTime(0),mTotalTime(0),uiFPSProgram(0)
    {
        pFont = new Font();
        pFont->loadFont(fontBuf,bufSize, 32);
        loadFPSShader(pShaderProgram);
#ifdef _WIN32
        QueryPerformanceFrequency(&f);
        dqFreq = (double)f.QuadPart;
#endif

#if WRITEFPS2FILE
        FPSFile.open(FPS_FILE);
        timeCounter = 0;
#endif
    }
    ~FPSCounter()
    {
        if (NULL != pFont)
        {
            pFont->deleteFont();
            delete pFont;
        }
        glDeleteProgram(uiFPSProgram);
#if WRITEFPS2FILE
        FPSFile.close();
        timeCounter = 0;
#endif
    }

private:
    int getFPS();
    double getExecutionTime();
    Font* pFont;

    unsigned int uiFPSProgram;
    double mTotalTime;
#ifdef _WIN32
    double mLastTime;
    double mCurrentTime;
    double mDurationTime;
    double dqFreq;
    LARGE_INTEGER f;
#else
#ifdef _linux_
    struct timeval mLastTime;
    struct timeval mCurrentTime;
    double mDurationTime;
#else
    clock_t mLastTime;
    clock_t mCurrentTime;
    clock_t mDurationTime;
#endif
#endif

    int mFPS;

#if WRITEFPS2FILE
    ofstream FPSFile;
    int timeCounter;
#endif
};

#endif // ENABLE_FPS

#endif
