/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef LOADSCENE_H
#define LOADSCENE_H
#include <string>
#include <vector>
#include <boost/foreach.hpp>
#include <boost/property_tree/json_parser.hpp>
#include "Common.h"
#include "Timer.h"

using namespace glTF;

class Parser
{
public:
    glTFHandle* getFileNameInJson(const std::string& jsonFile);
    bool releaseFileName();
    int parseScene(Scene* pscene);
    bool parseJsonFile();
    void setJsonInfo(const std::string& sbuffer);
    void setJsonInfo(const std::string& direct, const std::string& fileName);
    Parser();
    ~Parser();
private:
    int readBuffers();
    bool parseNodes();
    bool parseMeshs();
    bool parseMaterials();
    bool parseCameras();
    bool parseLights();
    bool parseAttributes();
    int parseTechniques();
    bool parseAnim();
    bool parseSkins();

    bool parsePrimitive(const boost::property_tree::ptree& pTree, Mesh* pMesh);
    bool parseMaterialProper(const boost::property_tree::ptree& pTree,
                    Material* pMaterial,
                    const std::string& techniqueId);
    void parseNode(const std::string& id, Node* pParentNode,
                    const glm::mat4& pmatrix);
    int parseTechniqueProgram(const boost::property_tree::ptree& pInstanceTree,
                    const boost::property_tree::ptree& pParameterTree,
                    Technique* pTechnique);

    bool parseTechniqueState(boost::property_tree::ptree& pInstanceTree,
                    Technique* pTechnique);

    bool parseTechniqueLight(const boost::property_tree::ptree& pInstanceTree,
                    const boost::property_tree::ptree& pParameterTree,
                    Technique* pTechnique);

    std::string parseChannel(const boost::property_tree::ptree& pTree);
    int parsePara(const boost::property_tree::ptree& pTree, Animation* pAnimation,
                    const std::string& nodeId);
    bool inline isFoundPtree(boost::property_tree::ptree& pTree, const char* key)
    {
        if (pTree.find(key) != pTree.not_found())
        {
            return true;
        }
        return false;
    };
    boost::property_tree::ptree ptParse;
    Scene* pScene;
    std::string json_file;
    std::string json_directory;
    std::string json_buffer;
    bool is_json_in_buffer;
#if ENABLE_TIMER
    Timer* lTimer;
#endif
};

#endif