/*
* This file is part of the LibreOffice project.
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

#include "Timer.h"

int Timer::flag = 0;
int Timer::startKey = 0;
int Timer::endKey = 0;
static int check = 0;

Timer::Timer()
{
    if(flag == 0)
    {
        initTimer();
        initModels();
    }
    flag++;
}

Timer::~Timer()
{
#if WRITETIME2FILE
    Time::iterator i;
    Time::iterator j;
    funcList::iterator k;
    funcList::iterator n;

#if DETAILED_INFO
    modelList::iterator l;
    for(i = startTimeList.begin(), j = endTimeList.begin(), k = functionList.begin(),
        l = totalModelList.begin();i != startTimeList.end(), j != endTimeList.end(),
        k!= functionList.end(),l != totalModelList.end(); i++, j++, k++, l++)
    {
#if _WIN32
        timerFile << l->second << " : " << k-> second <<" Time Taken : "
                  << std::fixed << ((j->second - i->second) / dqFreq) * 1000 << " ms" << endl;
#else
        timerFile << l->second <<" : "<<<<k-> second <<" Time Taken : "
                  << std::fixed << (j->second - i->second) * 1000 << " ms" << endl;
#endif
    }
#endif

    /* Calculating the total time taken by each sections */
    for (n = funcNamesList.begin(); n != funcNamesList.end(); n++)
    {
        double totalTime = 0;
        for (k = functionList.begin(), i = startTimeList.begin(),j = endTimeList.begin();
             k != functionList.end(), i != startTimeList.end(), j != endTimeList.end();
             k++, i++, j++)
        {
            if (n->second == k->second)
#if _WIN32
                totalTime = totalTime + (((j->second - i->second) / dqFreq) * 1000);
#else
                totalTime = totalTime + ((j->second - i->second) * 1000);
#endif
        }
        timerFile << "Total time : " << std::fixed << n->second << " " << totalTime << " ms" <<endl;
    }
    timerFile.close();
#endif
}

void Timer::initModels()
{
#if ENABLE_TIMER
    int i = 0;
    std::ifstream fin(CON_FILE, std::ios::in);
    char line[1024] = {0};
    while (fin.getline(line, sizeof(line)))
    {
        modelName = line;
        std::size_t found = modelName.find("#");
        if(found)
        {
            modelsList[i] = modelName.substr(modelName.find_last_of("/") + 1,
                                    modelName.find_last_of("."));
            i++;
        }
    }
#endif
}

void Timer::initTimer()
{
#ifdef _WIN32
    QueryPerformanceCounter(&temp);
    dqFreq = (double)temp.QuadPart;
#endif
#if WRITETIME2FILE
    timerFile = ofstream(TIMER_FILE);
#endif
}

void Timer::startTimer(string funcName, int modelNum)
{
    funcList::iterator i;
    bool insert = 0;

    /* Set model num based on the counter value received to 
       select the model name from the models list */
    if(modelNum >= (int)modelsList.size())
        modelNum = modelNum % (int)modelsList.size();

    /* adding function names to the funcNamesList */
    if (check == 0)
    {
        i = funcNamesList.begin();
        funcNamesList.insert(std::pair<int, std::string>(check, funcName));
        check++;
    }
    else
    {
        for (i = funcNamesList.begin(); i != funcNamesList.end(); ++i)
        {
            if (i->second == funcName)
                insert = 1;
        }
        if (insert == 0)
            funcNamesList.insert(std::pair<int, std::string>(check, funcName));
        check++;
    }

    functionList.insert(std::pair<int, std::string>(startKey, funcName));
    totalModelList.insert(std::pair<int, std::string>(startKey,
                                                      modelsList[modelNum]));
#ifdef _WIN32
    _LARGE_INTEGER tmp;
    QueryPerformanceCounter(&tmp);
    startTimeList.insert(std::pair<int, double>(startKey++, (double)tmp.QuadPart));
#else
    double startTime;
#ifdef _linux_
    gettimeofday(&startTime, NULL);
    startTimeList.insert(std::pair<int, double>(startKey++, startTime));
#else
    startTime = clock();
    startTimeList.insert(std::pair<int, double>(startKey++, startTime));
#endif
#endif
}

void Timer::stopTimer()
{
#ifdef _WIN32
    _LARGE_INTEGER tmp;
    QueryPerformanceCounter(&tmp);
    endTimeList.insert(std::pair<int, double>(endKey++, (double)tmp.QuadPart));
#else
    double endTime;
#ifdef _linux_
    gettimeofday(&endTime, NULL);
    endTimeList.insert(std::pair<int, double>(endKey++, endTime));
#else
    endTime = clock();
    endTimeList.insert(std::pair<int, double>(endKey++, endTime));
#endif
#endif
}
