/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef TIMER_H
#define TIMER_H
#include "time.h"
#include "Common.h"
#ifdef _WIN32
#include <windows.h>
#endif

#ifdef _linux_
#include <sys/time.h>
#endif

#ifndef ENABLE_TIMER
#define ENABLE_TIMER 0
#endif

#define DETAILED_INFO 0

#if ENABLE_TIMER
#define WRITETIME2FILE 1
#define TIMER_FILE "./Time.txt"
#define CON_FILE   "config.txt"
#endif

#if WRITETIME2FILE
#include <iostream>
#include <iomanip>
#include <fstream>
#endif

#ifdef _WIN32
static double dqFreq;
static LARGE_INTEGER temp;
#endif

#ifdef WRITETIME2FILE
static ofstream timerFile;
#endif

typedef std::map<int, double> Time;
typedef std::map<int, std::string> funcList;
typedef std::map<int, std::string> modelList;

static Time startTimeList;
static Time endTimeList;
static funcList functionList;
static funcList funcNamesList;
static modelList modelsList;
static modelList totalModelList;

class Timer
{
public :
    Timer();
    ~Timer();

    /* Timer Functions */
    void initTimer();
    void initModels();
    void startTimer(string funcName, int modelId);
    void stopTimer();

public :
    static int startKey;
    static int endKey;
    std::string modelName;
    static int flag;
    static int modelNum;
};
#endif
