/*
* This file is part of the LibreOffice project.
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
#include "libgltf.h"
#include "RenderScene.h"

extern "C"
glTFHandle* gltf_renderer_init(const glTFFile* jsonfile)
{
    RenderScene* renderScene = new RenderScene();
    glTFHandle* gltf_result = renderScene->initScene(jsonfile);
    gltf_result->renderer = renderScene;
    return gltf_result;
}

extern "C"
int gltf_renderer_set_content(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->initRender(handle);
}

void gltf_render_FPS_initialize(glTFHandle* handle
                 ,unsigned char* fontBuf,unsigned int bufSize)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->initFPS(fontBuf,bufSize);
}

extern "C"
void gltf_prepare_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->prepareRender(&handle->viewport);
    return;
}

extern "C"
void gltf_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->render();
    return;
}

extern "C"
void gltf_complete_renderer(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->completeRender();
    return;
}

extern "C"
void gltf_renderer_release(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->releaseRender();
    delete renderScene;
    return;
}

glm::vec3* gltf_get_model_center_pos(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getModelCenterPos();
}

void gltf_get_camera_pos(glTFHandle* handle, glm::vec3* pos, glm::vec3* view, glm::vec3* up)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCameraPos(pos, view, up);
    return;
}

extern "C"
double gltf_get_model_size(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getModelSize();
}

extern "C"
void gltf_renderer_move_camera(glTFHandle* handle, double x, double y, double z, double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->moveCamera(x, y, z, time);
    return;
}

extern "C"
void gltf_renderer_rotate_camera(glTFHandle* handle, double horizontal, double vertical,
                                 double planar, double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->rotateCamera(horizontal, vertical, planar, time);
    return;
}

void gltf_renderer_rotate_model(glTFHandle* handle, double horizontal, double vertical,
                                 double planar)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->getCamera()->RotateObjectMouse(horizontal, vertical, planar);
    return;
}
extern "C"
void gltf_enable_rotation(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->EnableRotation();
}

extern "C"
void gltf_disable_rotation(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->DisableRotation();
}

extern "C"
void gltf_renderer_get_bitmap(glTFHandle** handle, int size,
                              char* buffer, GLenum format)
{
    if(size <= 0)
        return;
    RenderScene* renderScene = (RenderScene*)handle[0]->renderer;
    renderScene->prepareRenderBitmap(&(handle[0]->viewport));
    for (int i = 0; i < size; i++)
    {
        RenderScene* tRenderScene = (RenderScene*)handle[i]->renderer;
        tRenderScene->renderBitmap();
    }
    renderScene->completeRenderBitmap(&(handle[0]->viewport),
                                      (unsigned char*)buffer, format);
    return;
}

extern "C"
void gltf_animation_start(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->startAnimation();
    return;
}

extern "C"
void gltf_animation_stop(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->stopAnimation();
    return;
}

extern "C"
int gltf_animation_is_playing(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->isAnimPlay();
}

extern "C"
void gltf_animation_set_looping(glTFHandle* handle, int loop)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->setAnimLoop(loop);
    return;
}

extern "C"
int gltf_animation_get_looping(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimLoop();
}

extern "C"
double gltf_animation_get_duration(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimDuration();
}

extern "C"
void gltf_animation_set_time(glTFHandle* handle, double time)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    renderScene->setAnimTime(time);
    return;
}

extern "C"
double gltf_animation_get_time(glTFHandle* handle)
{
    RenderScene* renderScene = (RenderScene*)handle->renderer;
    return renderScene->getAnimTime();
}

